@@ -25,10 +25,10 @@
 #include "expression.h"
 #include "analitzautils.h"
 #include "operations.h"
-#include <localize.h>
 #include "apply.h"
 #include "value.h"
 #include "matrix.h"
+#include <QCoreApplication>
 
 QDebug operator<<(QDebug dbg, const Analitza::ExpressionType &c)
 {
@@ -196,36 +196,69 @@ ExpressionType ExpressionTypeChecker::solve(const Operator* o, const QVector< Ob
 						Q_ASSERT(!opt.parameters().last().isError());
 						QMap<int, ExpressionType> starToType, starToParam;
 						
-						ExpressionType first =_first .starsToType(_starToType);
-						ExpressionType second=_second.starsToType(_starToType);
-// 						qDebug() << "9999999" << _first.assumptions() << first.assumptions() << starToType;
+						bool valid = false;
+						QMap<QString, ExpressionType> assumptions;
 						
-						starToType=ExpressionType::computeStars(starToType, first,  opt.parameters()[0]);
-						starToType=ExpressionType::computeStars(starToType, second, opt.parameters()[1]);
-						
-						first =first .starsToType(starToType);
-						second=second.starsToType(starToType);
-						
-						starToParam=ExpressionType::computeStars(starToParam, opt.parameters()[0], first);
-						starToParam=ExpressionType::computeStars(starToParam, opt.parameters()[1], second);
-// 						qDebug() << "XXXXXX" << starToParam;
-// 						qDebug() << "PPPPPP" << opt << first << second << "|||||" << first.assumptions() << second.assumptions();
-						
-// 						starToType=ExpressionType::computeStars(starToType, first,  opt.parameters()[0].starsToType(starToParam));
-// 						starToType=ExpressionType::computeStars(starToType, second, opt.parameters()[1].starsToType(starToParam));
-						
-// 						first =first .starsToType(starToType);
-// 						second=second.starsToType(starToType);
-						
-						QMap<QString, ExpressionType> assumptions=first.assumptions();
-						bool valid=ExpressionType::assumptionsMerge(assumptions, second.assumptions());
-						
-// 						qDebug() << "fifuuuuuuu" << first << (it-1)->toString() << 
-// 													second << it->toString() << assumptions << valid;
-						
-						valid &= !first.isError() && !second.isError();
-						valid &= first .canReduceTo(opt.parameters()[0].starsToType(starToParam));
-						valid &= second.canReduceTo(opt.parameters()[1].starsToType(starToParam));
+						//NOTE In matrix multiplication (A*B) A may or not be reduced to B, since is allowed 
+						//different matrix sizes provided that are valid sizes for the multiplication, 
+						//thus we need a special case for matrix type
+						if (current.type() == ExpressionType::Matrix) {
+							ExpressionType first =_first .starsToType(_starToType);
+							ExpressionType second=_second.starsToType(_starToType);
+							
+	// 						qDebug() << "9999999" << _first.assumptions() << first.assumptions() << starToType;
+							
+							starToType=ExpressionType::computeStars(starToType, first,  opt.parameters()[0]);
+							first =first .starsToType(starToType);
+							starToType=ExpressionType::computeStars(starToType, second, opt.parameters()[1]);
+							second=second.starsToType(starToType);
+							
+							assumptions=first.assumptions();
+							valid=ExpressionType::assumptionsMerge(assumptions, second.assumptions());
+							
+	// 						qDebug() << "PPPPPP" << opt << first << second << "|||||" << first.assumptions() << second.assumptions();
+							
+							valid &= !first.isError() && !second.isError();
+							starToParam=ExpressionType::computeStars(starToParam, opt.parameters()[0], first);
+	// 						qDebug() << "XXXXXX1" << starToParam;
+							valid &= first .canReduceTo(opt.parameters()[0].starsToType(starToParam));
+							starToParam=ExpressionType::computeStars(starToParam, opt.parameters()[1], second);
+	// 						qDebug() << "XXXXXX2" << starToParam;
+							valid &= second.canReduceTo(opt.parameters()[1].starsToType(starToParam));
+						} else {
+							ExpressionType first =_first .starsToType(_starToType);
+							ExpressionType second=_second.starsToType(_starToType);
+	// 						qDebug() << "9999999" << _first.assumptions() << first.assumptions() << starToType;
+							
+							starToType=ExpressionType::computeStars(starToType, first,  opt.parameters()[0]);
+							starToType=ExpressionType::computeStars(starToType, second, opt.parameters()[1]);
+							
+							first =first .starsToType(starToType);
+							second=second.starsToType(starToType);
+							
+							starToParam=ExpressionType::computeStars(starToParam, opt.parameters()[0], first);
+							starToParam=ExpressionType::computeStars(starToParam, opt.parameters()[1], second);
+	// 						qDebug() << "XXXXXX" << starToParam;
+	// 						qDebug() << "PPPPPP" << opt << first << second << "|||||" << first.assumptions() << second.assumptions();
+							
+	// 						starToType=ExpressionType::computeStars(starToType, first,  opt.parameters()[0].starsToType(starToParam));
+	// 						starToType=ExpressionType::computeStars(starToType, second, opt.parameters()[1].starsToType(starToParam));
+							
+	// 						first =first .starsToType(starToType);
+	// 						second=second.starsToType(starToType);
+							
+							assumptions=first.assumptions();
+							valid=ExpressionType::assumptionsMerge(assumptions, second.assumptions());
+							
+	// 						qDebug() << "fifuuuuuuu" << first << (it-1)->toString() << 
+	// 													second << it->toString() << assumptions << valid;
+							
+							valid &= !first.isError() && !second.isError();
+							valid &= first .canReduceTo(opt.parameters()[0].starsToType(starToParam));
+							valid &= second.canReduceTo(opt.parameters()[1].starsToType(starToParam));
+							
+	// 						qDebug() << "POPOPO" << (it-1)->toString() << it->toString() << valid << first << second << starToParam;
+						}
 						
 // 						qDebug() << "POPOPO" << (it-1)->toString() << it->toString() << valid << first << second << starToParam;
 						if(valid) {
@@ -264,7 +297,7 @@ ExpressionType ExpressionTypeChecker::solve(const Operator* o, const QVector< Ob
 				}
 			}
 			if(!found) {
-				addError(i18n("Could not find a type that unifies '%1'", o->toString()));
+				addError(QCoreApplication::tr("Could not find a type that unifies '%1'").arg(o->toString()));
 			}
 // 			qDebug() << qPrintable("\\"+QString(--ind, '-')) << o->toString() << ret;
 		}
@@ -436,7 +469,7 @@ QVariant ExpressionTypeChecker::visit(const Apply* c)
 				tt=current.contained();
 				tt.addAssumptions(current.assumptions());
 			} else
-				addError(i18n("The domain should be either a vector or a list"));
+				addError(QCoreApplication::tr("The domain should be either a vector or a list"));
 			//TODO: should control the many case
 		}
 		
@@ -514,7 +547,6 @@ QVariant ExpressionTypeChecker::visit(const Apply* c)
 							if(error)
 								break;
 						}
-						
 					}
 					returntype.addAssumptions(type.assumptions());
 					
@@ -522,13 +554,14 @@ QVariant ExpressionTypeChecker::visit(const Apply* c)
 				}
 				
 				if(error) {
-// 					addError(i18n("Cannot call '%1'", c->m_params.first()->toString()));
+					addError(QCoreApplication::translate("type error message", "Cannot call '%1'").arg(c->m_params.first()->toString()));
 					current=ExpressionType(ExpressionType::Error);
 				} else
 					current=ret2;
 			} else if(returned.isError()) {
 				current=ExpressionType(ExpressionType::Error);
-			} else {
+			}
+			else {
 				ExpressionType ret(ExpressionType::Many), signature(returned);
 				
 				QList<ExpressionType> alts=signature.type()==ExpressionType::Many ? signature.alternatives() : QList<ExpressionType>() << signature;
@@ -540,33 +573,64 @@ QVariant ExpressionTypeChecker::visit(const Apply* c)
 						continue;
 					}
 					
+					const bool isvariadic = (opt.parameters().first().type() == ExpressionType::Any);
+					//if we have a variadic check only the return type (that is why we put 0 as args count here)
+					const int nargs = opt.parameters().size()-1;
+					
 					foreach(const ExpressionType& f, args) {
 						QList<ExpressionType> altargs=f.parameters();
 						
-						if(opt.parameters().size()!=altargs.size()+1) {
-							addError(i18np("Invalid parameter count for '%2'. Should have 1 parameter.",
-														"Invalid parameter count for '%2'. Should have %1 parameters.",
-														opt.parameters().size()-1, c->toString()));
+						bool valid=true;
+						
+						if (isvariadic) { // if is variadic don't check the number of args
+							Q_ASSERT(opt.parameters().size() == 2); // Any and return type
+							
+							//when use Any with contained we need to check if all args must be of the contained type
+							if (opt.parameters().first().hasContained()) {
+								for(int i=0; valid && i<altargs.size(); i++) {
+									if(!altargs[i].canReduceTo(opt.parameters().first().contained())) {
+										addError(QCoreApplication::tr("Cannot reduce '%1' to '%2'. Invalid type of parameter '%3' for '%4'").
+										arg(altargs[i].toString()).
+										arg(opt.parameters().first().contained().toString()).
+										arg(i+1).
+										arg(c->toString()));
+										
+										valid=false;
+										break;
+									}
+								}
+							}
+						} else if(opt.parameters().size()!=altargs.size()+1) {
+							addError(QCoreApplication::tr("Invalid parameter count for '%2'. Should have %1 parameters.").arg(opt.parameters().size()-1).arg(c->toString()));
 							exit=true;
 							break;
 						}
 						
-						bool valid=true;
 						QMap<QString, ExpressionType> assumptions;
 						QMap<int, ExpressionType> starToType, starToParam;
-						for(int i=0; valid && i<opt.parameters().size()-1; i++) {
-							if(!altargs[i].canCompareTo(opt.parameters()[i])) {
-								valid=false;
-								break;
+						
+						for(int i=0; valid && i<nargs; i++) {
+							if (i<opt.parameters().size() && i<altargs.size()) {
+								if(!altargs[i].canCompareTo(opt.parameters()[i])) {
+									if (isvariadic)
+										addError(QCoreApplication::tr("Cannot compare '%1' to '%2'. Invalid type of parameter '%3' for '%4'").
+										arg(altargs[i].toString()).
+										arg(opt.parameters()[i].toString()).
+										arg(i+1).
+										arg(c->toString()));
+									
+									valid=false;
+									break;
+								}
+								
+								starToType=ExpressionType::computeStars(starToType, opt.parameters()[i], altargs[i]);
+								
+								ExpressionType t=opt.parameters()[i].starsToType(starToType);
+								starToParam=ExpressionType::computeStars(starToParam, altargs[i], t);
+								valid=!t.isError() && inferType(altargs[i].starsToType(starToParam), t, &assumptions);
+								
+// 								qDebug() << "pepeluis" << c->toString() << c->m_params[i+1]->toString() << altargs[i] << t << valid;
 							}
-							
-							starToType=ExpressionType::computeStars(starToType, opt.parameters()[i], altargs[i]);
-							
-							ExpressionType t=opt.parameters()[i].starsToType(starToType);
-							starToParam=ExpressionType::computeStars(starToParam, altargs[i], t);
-							valid=!t.isError() && inferType(altargs[i].starsToType(starToParam), t, &assumptions);
-							
-// 							qDebug() << "pepeluis" << c->toString() << c->m_params[i+1]->toString() << altargs[i] << t << valid;
 						}
 						
 						valid &= ExpressionType::matchAssumptions(&starToType, opt.parameters().last().assumptions(), assumptions);
@@ -583,12 +647,12 @@ QVariant ExpressionTypeChecker::visit(const Apply* c)
 					if(exit)
 						break;
 				}
-					
+				
 				if(ret.alternatives().isEmpty()) {
 // 					qDebug() << "peee" << c->toString() << signature << exps;
 					
 					current=ExpressionType(ExpressionType::Error);
-					addError(i18n("Could not call '%1'", c->toString()));
+					addError(QCoreApplication::tr("Could not call '%1'").arg(c->toString()));
 				} else
 					current=ret;
 			}
@@ -596,7 +660,7 @@ QVariant ExpressionTypeChecker::visit(const Apply* c)
 		default:
 			current=solve(&o, c->values());
 			if(current.type()==ExpressionType::Error)
-				addError(i18n("Could not solve '%1'", c->toString()));
+				addError(QCoreApplication::tr("Could not solve '%1'").arg(c->toString()));
 			break;
 	}
 	
@@ -629,7 +693,7 @@ ExpressionType ExpressionTypeChecker::tellTypeIdentity(const QString& name, cons
 			for(QList< ExpressionType >::iterator itf=optsFound.begin(), itfEnd=optsFound.end(); itf!=itfEnd; ++itf) {
 				if(!itf->canReduceTo(type)) {
 // 					qDebug() << "incoherent type" << *itf << type;
-					addError(i18n("Incoherent type for the variable '%1'", name));
+					addError(QCoreApplication::tr("Incoherent type for the variable '%1'").arg(name));
 					break;
 				}
 				QMap<int, ExpressionType> stars;
@@ -652,7 +716,7 @@ QVariant ExpressionTypeChecker::visit(const Container* c)
 			ExpressionType type=commonType(c->m_params);
 			
 			if(type.isError()) {
-				addError(i18n("Could not determine the type for piecewise"));
+				addError(QCoreApplication::tr("Could not determine the type for piecewise"));
 				current=type;
 			} else {
 				QList<ExpressionType> alts=type.type()==ExpressionType::Many ? type.alternatives() : QList<ExpressionType>() << type, alts2;
@@ -715,7 +779,6 @@ QVariant ExpressionTypeChecker::visit(const Container* c)
 			m_lambdascope+=bvars.toSet();
 			c->m_params.last()->accept(this);
 			m_lambdascope=aux;
-			QMap<QString, ExpressionType> assumptions=current.assumptions();
 			
 			QList<ExpressionType> alts=current.type()==ExpressionType::Many ? current.alternatives() : QList<ExpressionType>() << current;
 			ExpressionType res=ExpressionType(ExpressionType::Many);
@@ -806,7 +869,36 @@ QVariant ExpressionTypeChecker::visit(const Vector* l)
 
 QVariant ExpressionTypeChecker::visit(const Matrix* m)
 {
-	visitListOrVector(m, ExpressionType::Matrix, m->size());
+	const Matrix* v = m;
+	ExpressionType::Type t = ExpressionType::Matrix;
+	int size = m->rowCount();
+	
+	QList<Object*> values;
+	foreach(MatrixRow *row, m->rows())
+		values.append(row);
+	
+	ExpressionType cont=commonType(values);
+	
+	if(cont.type()==ExpressionType::Many) {
+		ExpressionType toret(ExpressionType::Many);
+		foreach(const ExpressionType& contalt, cont.alternatives()) {
+			QMap<QString, ExpressionType> assumptions;
+			assumptions=typeIs(v->constBegin(), v->constEnd(), contalt);
+			ExpressionType cc(t, contalt, size);
+			
+			bool b=ExpressionType::assumptionsMerge(cc.assumptions(), assumptions);
+			if(b)
+				toret.addAlternative(cc);
+		}
+		
+		current=toret;
+	} else if(!cont.isError()) {
+		QMap<QString, ExpressionType> assumptions=typeIs(v->constBegin(), v->constEnd(), cont);
+		current=ExpressionType(t, cont, size);
+		current.addAssumptions(assumptions);
+	} else
+		current=ExpressionType(ExpressionType::Error);
+	
 	return QString();
 }
 
@@ -843,7 +935,7 @@ QMap<QString, ExpressionType> ExpressionTypeChecker::typeIs(T it, const T& itEnd
 	}
 	
 	if(!valid)
-		addError(i18n("Unexpected type"));
+		addError(QCoreApplication::tr("Unexpected type"));
 	return ret;
 }
 
@@ -866,7 +958,7 @@ QMap<QString, ExpressionType> ExpressionTypeChecker::typeIs(const Object* o, con
 	}
 	
 	if(!corr)
-		addError(i18n("Cannot convert '%1' to '%2'", o->toString(), type.toString()));
+		addError(QCoreApplication::tr("Cannot convert '%1' to '%2'").arg(o->toString()).arg(type.toString()));
 	
 	return assumptions;
 }
@@ -912,9 +1004,16 @@ void ExpressionTypeChecker::addError(const QString& err)
 	m_err.last().append(err);
 }
 
+void ExpressionTypeChecker::initializeVars(const QMap< QString, ExpressionType >& types)
+{
+	m_vars=types;
+}
+
 QMap<QString, ExpressionType> ExpressionTypeChecker::variablesTypes() const
 {
 	return m_vars;
 }
 
+QVariant ExpressionTypeChecker::visit(const None* ) { Q_ASSERT(false && "should not get here"); return QVariant(); }
+
 }
