@@ -27,7 +27,7 @@
 #include <cmath>
 #include <QPalette>
 #include <QPainter>
-#include <cmath>
+#include <qfontdatabase.h>
 #include <QDebug>
 #include <qnumeric.h>
 
@@ -67,7 +67,7 @@ struct Plotter2D::GridInfo
 Plotter2D::Plotter2D(const QSizeF& size)
     : m_showGrid(true)
     , m_showMinorGrid(false)
-    , m_gridColor(Qt::lightGray)
+    , m_gridColor(QColor(Qt::lightGray).lighter(120))
     , m_backgroundColor(Qt::white)
     , m_autoGridStyle(true)
     , m_gridStyleHint(Squares)
@@ -102,7 +102,7 @@ void Plotter2D::setGridStyleHint(GridStyle suggestedgs)
 const QColor Plotter2D::computeSubGridColor() const
 {
     //impl. details: since any kde user can create any palette style, we need this hard/magic numbres
-    // becuase there is no way to guess, however, this code covers almost any case, 
+    // because there is no way to guess, however, this code covers almost any case, 
     // it was tested with more then 35 color styles, and all give good results.
     
     QColor col = m_gridColor;
@@ -303,9 +303,9 @@ const QString Plotter2D::computeAngleLabelByFrac(unsigned int n, unsigned int d)
     {
         case Radian:
         {
-            s = (n==1) ? QString("") : QString::number(n);
+            s = (n==1) ? QStringLiteral("") : QString::number(n);
             s += PiSymbol;
-            s += (d==1) ? QString("") : "/"+QString::number(d);
+            s += (d==1) ? QStringLiteral("") : "/"+QString::number(d);
         }
         break;
         case Degree: s = QString::number(radiansToDegrees(n*M_PI/d))+DegreeSymbol; break;
@@ -350,12 +350,14 @@ void Plotter2D::drawCartesianTickLabels(QPainter* painter, const Plotter2D::Grid
     
     QString s;
     QPointF p;
-
-    painter->setPen(QPen(QPalette().text().color()));
     
     int from = (axis == Analitza::XAxis) ? gridinfo.nxinilabels : gridinfo.nyinilabels;
     int to = (axis == Analitza::XAxis) ? gridinfo.nxendlabels : gridinfo.nyendlabels;
     
+    painter->save();
+    QFont tickFont = QFontDatabase::systemFont(QFontDatabase::SmallestReadableFont);
+    painter->setFont(tickFont);
+    painter->setPen(QPen(QPalette().text().color()));
     for (int i = from; i <= to; ++i)
     {
         if (i == 0) continue;
@@ -374,7 +376,7 @@ void Plotter2D::drawCartesianTickLabels(QPainter* painter, const Plotter2D::Grid
                 break;
             case Trigonometric:
             {
-                s = (i < 0) ? "-" : "";
+                s = (i < 0) ? QString(QLatin1Char('-')) : QString();
                 
                 if (incbig)
                     s += computeAngleLabelByStep(qAbs(i), bigstep);
@@ -395,15 +397,14 @@ void Plotter2D::drawCartesianTickLabels(QPainter* painter, const Plotter2D::Grid
         else
             painter->drawText(p.x()-swidth-axisyseparation, p.y()+hfmhalf-textposcorrection, s);
     }
+    painter->restore();
 }
 
 void Plotter2D::drawPolarTickLabels(QPainter* painter, const Plotter2D::GridInfo& gridinfo) const
 {
-    QString s;
-    QPointF p;
     unsigned int k = 1;
 
-    painter->setPen(QPen(QPalette().color(QPalette::Disabled, QPalette::Text))); // polar ticks color
+    painter->setPen(m_gridColor);
     
     //TODO if minor
     const double newinc = gridinfo.inc/(gridinfo.sub5 ? 5 : 4); // inc with sub intervals
@@ -472,12 +473,8 @@ void Plotter2D::drawPolarTickLabels(QPainter* painter, const Plotter2D::GridInfo
     {
         const QPair<unsigned int, unsigned int> frac = simplifyFraction(k, halfturns);
         
-        if (k != 0)
-            s = computeAngleLabelByFrac(frac.first, frac.second);
-        else
-            s = "0";
-        
-        p = QPointF(r*std::cos(t), r*std::sin(t));
+        QString s = (k != 0) ? computeAngleLabelByFrac(frac.first, frac.second) : QStringLiteral("0");
+        QPointF p(r*std::cos(t), r*std::sin(t));
         
         if (viewport.contains(p))
         {
@@ -816,6 +813,9 @@ void Plotter2D::drawFunctions(QPaintDevice *qpd)
     
     int current=currentFunction();
 
+    int dpr = qMax<int>(1, qRound(qpd->logicalDpiX()/100.));
+    dpr *= dpr;
+
     for (int k = 0; k < m_model->rowCount(); ++k )
     {
         PlaneCurve* curve = dynamic_cast<PlaneCurve *>(itemAt(k));
@@ -825,8 +825,8 @@ void Plotter2D::drawFunctions(QPaintDevice *qpd)
             continue;
 
         pfunc.setColor(curve->color());
-        pfunc.setWidth((k==current)+1);
-//         pfunc.setStyle(Qt::SolidLine);
+        pfunc.setWidth((k==current)+dpr);
+        pfunc.setStyle(Qt::SolidLine);
         p.setPen(pfunc);
 
         const QVector<QPointF> &vect=curve->points();
